package widgets

import (
	"fmt"
	"github.com/awesome-gocui/gocui"
	"strings"
)

const (
	StatusInformation int = iota
	StatusSuccess
	StatusAlert
)

type Window struct {
	Widget
	currentHeight    int
	children         []*WidgetInterface
	CurrentSelection string
	Status           string
	StatusType       int
	status           *gocui.View
}

func NewWindow(g *gocui.Gui, name string, width int) *Window {
	return &Window{
		Widget: Widget{
			gui:         g,
			name:        name,
			width:       width,
			elementType: "window",
		},
	}
}

func (w *Window) SetStatus(statusType int, text string) {
	if w.status != nil {
		w.gui.Update(func(g *gocui.Gui) error {
			switch statusType {
			case StatusAlert:
				w.status.BgColor = gocui.ColorRed
				w.status.FgColor = gocui.ColorBlack
			case StatusInformation:
				w.status.BgColor = gocui.ColorBlue
				w.status.FgColor = gocui.ColorWhite
			case StatusSuccess:
				w.status.BgColor = gocui.ColorGreen
				w.status.FgColor = gocui.ColorWhite
			}
			w.status.Clear()
			_, _ = fmt.Fprint(w.status, text)
			return nil
		})
	}
}

func (w *Window) GetForm() map[string]interface{} {
	form := map[string]interface{}{}
	for _, child := range w.children {
		if (*child).GetBuffer() != "" {
			val := strings.TrimSpace((*child).GetBuffer())
			if (*child).GetElementType() == "checkbox" {
				if val == "x" {
					form[(*child).GetName()] = true
				} else {
					form[(*child).GetName()] = false
				}
			} else {
				form[(*child).GetName()] = val
			}
		}
	}
	return form
}

func (w *Window) AddWidget(element *WidgetInterface) {
	if element != nil {
		w.children = append(w.children, element)
	}
}

func (w *Window) Deactivate() {
	for _, child := range w.children {
		(*child).Deactivate()
	}
	w.gui.DeleteView(w.view.Name())
}

func (w *Window) Rotator(g *gocui.Gui, v *gocui.View) error {
	next := false
	for _, child := range w.children {
		if (*child).IsRotatable() && (*child).Visible() {
			(*child).Deactivate()
		}
	}
	for _, child := range w.children {
		if (*child).Visible() {
			if next && (*child).IsRotatable() {
				(*child).Activate()
				w.CurrentSelection = (*child).GetName()
				return nil
			}

			if (*child).GetName() == w.CurrentSelection {
				next = true
			}
		}
	}

	for _, child := range w.children {
		if (*child).IsRotatable() && (*child).Visible() {
			(*child).Activate()
			w.CurrentSelection = (*child).GetName()
			return nil
		}
	}
	return nil
}

func (w *Window) RotatorBack(g *gocui.Gui, v *gocui.View) error {
	var back *WidgetInterface

	for _, child := range w.children {
		if (*child).IsRotatable() && (*child).Visible() {
			(*child).Deactivate()
		}
	}
	for _, child := range w.children {
		if (*child).Visible() {
			if (*child).IsRotatable() && (*child).GetName() != w.CurrentSelection {
				back = child
			}

			if (*child).GetName() == w.CurrentSelection {
				if back != nil {
					(*back).Activate()
					w.CurrentSelection = (*back).GetName()
					return nil
				}
			}
		}
	}
	if back != nil {
		(*back).Activate()
		w.CurrentSelection = (*back).GetName()
		return nil
	}

	return nil
}

func (w *Window) Layout(g *gocui.Gui) (err error) {
	w.currentHeight = 0
	totalHeight := 0
	for _, child := range w.children {
		if (*child).(WidgetInterface).Visible() {
			totalHeight += (*child).(WidgetInterface).GetHeight() + (*child).(WidgetInterface).GetPadding() - 1
		}
	}
	totalHeight += 1
	maxX, maxY := g.Size()
	w.x = maxX/2 - w.width/2
	w.y = maxY/2 - totalHeight/2

	w.view, _ = g.SetView(w.name, w.x, w.y, w.x+w.width, w.y+totalHeight, 0)
	hasActive := false
	for _, child := range w.children {
		if (*child).(WidgetInterface).Visible() {
			(*child).SetParent(w)
			(*child).Layout(g)
			//w.currentHeight += (*child).GetHeight() + (*child).(WidgetInterface).GetPadding()
			w.currentHeight += (*child).GetHeight() + (*child).(WidgetInterface).GetPadding() - 1
			if !hasActive && (*child).IsRotatable() && w.CurrentSelection == "" {
				(*child).Activate()
				w.CurrentSelection = (*child).GetName()
				hasActive = true
			} else if w.CurrentSelection == (*child).GetName() {
				(*child).Activate()
				hasActive = true
			}
		} else {
			g.DeleteView((*child).GetName())
		}
	}

	// Always creating a one line status view at the top of the screen
	w.status, err = g.SetView("status", 2, 0, maxX-2, 3, 0)
	if err != nil {
		if err != gocui.ErrUnknownView {
			return err
		}
		w.status.Frame = false
		w.status.Editable = false
	}

	return nil
}
